from __future__ import print_function
import os
import sys
import subprocess
import urllib
import zipfile
import shutil
import json
from qgis.utils import *
from qgis.core import *
from qgis.gui import *
from PyQt4.QtSql import QSqlDatabase, QSqlQuery
from PyQt4.QtGui import *
from __builtin__ import True

class UpdateManager():

	def __init__(self, connection=None, version=None):

		self.connection = QSqlDatabase().addDatabase('QPSQL', 'updateDb') if connection == None else connection
		self.update_connection = self.connection.database(u'updateDb')
		self.update_table = "public.version_sw"
		self.current_version = version

		self.zipName = 'MS_Project.zip'
		self.download_dir = os.path.join(os.getcwd(), 'update_packages')
		#self.download_dir = os.path.join(os.environ['LOCALAPPDATA'], 'ms_project')
		self.download_package =  os.path.join(self.download_dir, self.zipName)


	def setConnection(self, host='localhost', port=5433, dbname='cnr', user='postgres', password='postgres'):
		self.update_connection.setHostName(host)
		self.update_connection.setPort(int(port))
		self.update_connection.setDatabaseName(dbname)
		self.update_connection.setUserName(user)
		self.update_connection.setPassword(password)


	def setUpdateTable(self, update_table):
		self.update_table = "public.{}".format(update_table)


	def setCurrentVersion(self, version):
		self.current_version = version


	def getCurrentVersion(self):
		return self.current_version


	def getUpdateTable(self):
		return self.update_table


	def getConnection(self):
		if self.update_connection.isValid():
			hostName = self.update_connection.hostName()
			port = self.update_connection.port()
			dbName = self.update_connection.databaseName()
			user = self.update_connection.userName()
			password = self.update_connection.password()

			if self.update_connection.isOpen():
				connection_status = 'Active'
			else:
				connection_status = 'Inactive'

			report = "Host: {}\nPort: {}\nDatabase: {}\nUser: {}\nPassword: {}\nStatus: {}".format(hostName, port, dbName, user, password, connection_status)

		else:
			report = "No connection - Set a valid connection"

		print(report)


	def checkUpdate(self):
		queryText = """http://150.146.31.30/ms3mon/php/getSwVersion.php?versione={}""".format(self.current_version)
		new_versions = json.loads(urllib.urlopen(queryText).read())
		if new_versions:
			links = []
			for single_version in new_versions:
				links.append(single_version["link"])
			return (True, links)
		else:
			return (False, None)



	def downloadPackage(self, link):
		try:
			os.mkdir(self.download_dir)
		except:
			pass
		try:
			result_download = urllib.urlretrieve(link, self.download_package)
			return (True, result_download[0])		# file path
		except ContentTooShortError:
			return (False, None)

	def unzipPackage(self):
		zip_file = zipfile.ZipFile(self.download_package, 'r')
		zip_file.extractall(path=self.download_dir)
		zip_file.close()


	def getNewFiles(self):
		newFiles = []
		for root, dirs, files in os.walk(self.download_dir):
			for singleFile in files:
				if singleFile.endswith('zip'):
					continue
				newFiles.append(os.path.join(root, singleFile))
		return newFiles

	def updateFiles(self):
		newFiles = self.getNewFiles()
		movedFiles = []
		for singleFile in newFiles:
			try:
				movedFile = shutil.move(singleFile, singleFile.replace(self.download_dir, os.path.dirname(os.getcwd())))
				movedFiles.append(True)
			except:
				movedFiles.append(False)
		if all(movedFiles):
			return True
		return False

	def clearTemp(self):
		for root, dirs, files in os.walk(self.download_dir):
			for singleFile in files:
				result = os.remove(os.path.join(root, singleFile))
		#shutil.rmtree(self.download_dir)
		shutil.rmtree(self.download_dir, ignore_errors=True)

	def errorMessage(self, errorText):
		msgBox = QMessageBox()
		msgBox.setText("ERRORE")
		msgBox.setStandardButtons(QMessageBox.Ok)
		msgBox.setDetailedText(errorText)
		return msgBox.exec_()


	def start(self):
		if self.update_connection.open():
			return True
		else:
			return False


	def stop(self):
		if self.update_connection.isOpen():
			self.update_connection.close()
			return 'Connection stopped'
		else:
			return 'Connection already stopped'


def installUpdate():
	msgBox = QMessageBox()
	msgBox.setIcon(1)
	msgBox.setWindowTitle(u"Update Manager")
	msgBox.setText(u"E' disponibile una nuova versione")
	msgBox.setInformativeText("NB: L'installazione richiedera' qualche istante\nAttendere la notifica di completamento")
	btnOk = QPushButton("Installa")
	btnCancel = QPushButton("Cancel")
	msgBox.addButton(btnOk, QMessageBox.AcceptRole)
	msgBox.addButton(btnCancel, QMessageBox.RejectRole)
	msgBox.setDefaultButton(btnOk)

	installConfirm = msgBox.exec_()

	return installConfirm


def messageBox(title, body, type):
	if type == 1:
		QMessageBox.information(iface.mainWindow(), title, body, QMessageBox.Ok, QMessageBox.Ok)
	elif type == 2:
		QMessageBox.warning(iface.mainWindow(), title, body, QMessageBox.Ok, QMessageBox.Ok)
	elif type == 3:
		QMessageBox.critical(iface.mainWindow(), title, body, QMessageBox.Ok, QMessageBox.Ok)


def updateProject(self, mode='manual'):
	mode = mode
	update = UpdateManager()
	update.setConnection()
	with open(r"./metadata/version") as f:
		update.setCurrentVersion(f.readlines()[0].strip())
	try:
		update.clearTemp()
	except:
		pass
	result, links = update.checkUpdate()
	if result:
		installConfirm = installUpdate()
		if installConfirm == QMessageBox.AcceptRole:
			msgBox = QMessageBox()
			msgBox.setWindowTitle("Aggiornamento")
			msgBox.setText("ATTENDERE\n\nAggiornamento in corso")
			msgBox.setStandardButtons(QMessageBox.NoButton)
			msgBox.show()
			QCoreApplication.processEvents()
			for link in links:
				result, package = update.downloadPackage(link)
				if result:
					update.unzipPackage()
				else:
					# download interrotto
					return
			result = update.updateFiles()
			msgBox.hide()
			del msgBox
			if result:
				messageBox("Stato Aggiornamento", "Il plugin e' stato aggiornato\n\n\nAttenzione: QGis verra' chiuso.\nE' necessario riavviare per utilizzare la nuova versione del plugin", 1)
				update.clearTemp()
				iface.actionExit().trigger()
			else:
				messageBox("Stato Aggiornamento", "WARNING - L'aggiornamento non e' andato a buon fine", 2)
				# errore installazione
				pass

		else:
			pass
	else:
		# errore nella query o plugin aggiornato
		if mode == 'manual':
			messageBox("Aggiornamento Plugin", "Il plugin in uso risulta aggiornato all'ultima versione disponibile", 1)
		pass

	update.clearTemp()



class infoDialog(QDialog):

	def __init__(self, title, text, informativeText, parent=None):
		super(infoDialog, self).__init__(parent)
		self.title = title
		self.text = text
		self.informativeText = informativeText

		text = QLabel(self.text)
		infoText = QLabel(self.informativeText)

		layout = QVBoxLayout()
		layout.addWidget(text)
		layout.addWidget(infoText)
		self.setLayout(layout)
		self.setWindowTitle(self.title)


if __name__ == "__main__":
	app = QApplication(sys.argv)
	infoDlg = infoDialog("Aggiornamenti", "ATTENDERE\n", "Aggiornamento in corso")
	infoDlg.show()
	sys.exit(app.exec_())

